<?php

namespace EventPortal\Domain\Util\Notification;

use EventPortal\Domain\Util\Notification\Models\EventNotification;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Notifications\Notification;
use Illuminate\Support\Str;

class SystemNotification extends Notification implements ShouldQueue
{
    use Queueable;

    public $subject;

    public $data;

    public $key;

    public $from;

    public $replyTo;

    /**
     * Create a new notification instance.
     *
     * @return void
     */
    public function __construct(
        string $key,
        NotificationData $data,
        ?string $subject = null,
        ?string $replyTo = null
    ) {
        $this->key = $key;
        $this->data = $data;
        $this->subject = $subject;
        $this->replyTo = $replyTo;
    }

    /**
     * Get the notification's delivery channels.
     * Could use $notifiable for user preferences (email/sms/database) in future
     *
     * @param  mixed  $notifiable
     * @return array
     */
    public function via($notifiable): array
    {
        if ($notifiable->delegate !== null && $this->isForAnEvent()) {
            $eventNotification = EventNotification::create([
                'delegate_id' => $notifiable->delegate->id,
                'event_id' => $this->data->getEvent()->id,
                'method' => 'email',
                'key' => $this->key,
                'status' => 'queued',
            ]);
            $this->data->setId($eventNotification->id);
        }

        return [
            'mail',
        ];
    }

    /**
     * Determine which queues should be used for each notification channel.
     *
     * @return array
     */
    public function viaQueues(): array
    {
        return [
            'mail' => 'email',
        ];
    }

    /**
     * Get the mail representation of the notification.
     *
     * @param  mixed  $notifiable
     */
    public function toMail($notifiable): MailMessage
    {
        $data = [
            'data' => $this->data,
            'recipient' => $notifiable,
        ];

        $email = (new MailMessage)
            ->subject($this->subject)
            ->view([
                "EventPortal::email.{$this->key}",
                "EventPortal::email.{$this->key}-plain",
            ], $data);

        if ($notifiable->delegate && $notifiable->delegate->hasPA()) {
            $email->cc($notifiable->email);
        }

        if ($this->replyTo) {
            $email->replyTo($this->replyTo);
        }

        return $email;
    }

    /**
     * Get the array representation of the notification.
     *
     * @param  mixed  $notifiable
     */
    // phpcs:disable Generic.CodeAnalysis.UnusedFunctionParameter
    public function toArray($notifiable): array
    {
        // phpcs:enable
        return [
            'data' => $this->data,
        ];
    }

    private function isForAnEvent(): bool
    {
        return Str::startsWith($this->key, 'event.');
    }
}
