<?php

namespace EventPortal\Tests\Unit\ActionList\Models;

use Carbon\Carbon;
use EventPortal\Domain\ActionList\Models\ActionList;
use EventPortal\Domain\ActionList\Models\Task;
use EventPortal\Domain\User\Models\User;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class TaskTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_get_related_action_list()
    {
        $actionList = factory(ActionList::class)->create();
        $task = factory(Task::class)->create(['action_list_id' => $actionList->id]);

        $this->assertSameModel($actionList, $task->actionList);
    }

    /** @test **/
    public function can_determine_if_completed()
    {
        $task = factory(Task::class)->state('completed')->create();

        $this->assertTrue($task->is_complete);
    }

    /** @test **/
    public function can_determine_not_completed()
    {
        $task = factory(Task::class)->create();

        $this->assertFalse($task->is_complete);
    }

    /** @test **/
    public function can_get_user_who_completed_task()
    {
        $user = factory(User::class)->create();
        $task = factory(Task::class)
            ->state('completed')
            ->create([
                'user_id' => $user->id,
            ]);

        $this->assertSameModel($user, $task->completed_by);
    }

    /** @test **/
    public function can_complete_task()
    {
        $user = factory(User::class)->create();
        $task = factory(Task::class)->create();

        $this->assertNull($task->completed_at);

        Carbon::setTestNow('23rd Feb 2021 08:33:00');

        $task->setAsComplete($user);

        $this->assertEquals('2021-02-23 08:33:00', $task->completed_at->format('Y-m-d h:i:s'));
        $this->assertEquals($user->id, $task->user_id);
    }

    /** @test **/
    public function can_mark_task_as_incomplete()
    {
        $user = factory(User::class)->create();
        $task = factory(Task::class)->state('completed')->create();

        $task->setAsIncomplete();

        $this->assertNull($task->completed_at);
        $this->assertNull($task->user_id);
    }

    /** @test **/
    public function can_toggle_task()
    {
        $user = factory(User::class)->create();
        $task = factory(Task::class)->create();

        $this->assertNull($task->completed_at);

        Carbon::setTestNow('23rd Feb 2021 08:33:00');

        $task->toggle($user);

        $this->assertEquals('2021-02-23 08:33:00', $task->completed_at->format('Y-m-d h:i:s'));
        $this->assertEquals($user->id, $task->user_id);

        $task->toggle($user);

        $this->assertNull($task->completed_at);
        $this->assertNull($task->user_id);
    }

    /** @test **/
    public function can_scope_to_completed_tasks()
    {
        $taskA = factory(Task::class)->create();
        $taskB = factory(Task::class)->state('completed')->create();

        $tasks = Task::completed()->get();

        $this->assertCount(1, $tasks);
        $this->assertContainsModel($taskB, $tasks);
        $this->assertNotContainsModel($taskA, $tasks);
    }

    /** @test **/
    public function can_scope_to_incompleted_tasks()
    {
        $taskA = factory(Task::class)->create();
        $taskB = factory(Task::class)->state('completed')->create();

        $tasks = Task::incomplete()->get();

        $this->assertCount(1, $tasks);
        $this->assertContainsModel($taskA, $tasks);
        $this->assertNotContainsModel($taskB, $tasks);
    }
}
