<?php

namespace EventPortal\Tests\Unit\Event\Form\Models;

use EventPortal\Domain\Component\Step;
use EventPortal\Domain\Event\Form\Models\Form;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Form\Models\FormStepSection;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Form\Models\QuestionOption;
use EventPortal\Domain\Event\Form\Models\QuestionValidation;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class FormStepTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function deletes_sections_questions_options_validations_when_deleting_step(): void
    {
        $step = factory(FormStep::class)->create();
        $section = factory(FormStepSection::class)->create(['form_step_id' => $step->id]);
        $question = factory(Question::class)->create(['form_step_section_id' => $section->id]);
        factory(QuestionOption::class)->create(['question_id' => $question->id]);
        factory(QuestionValidation::class)->create(['question_id' => $question->id]);

        $step->delete();

        $this->assertCount(0, FormStep::all());
        $this->assertCount(0, FormStepSection::all());
        $this->assertCount(0, Question::all());
        $this->assertCount(0, QuestionOption::all());
        $this->assertCount(0, QuestionValidation::all());
    }

    /** @test **/
    public function can_get_next_step(): void
    {
        $form = factory(Form::class)->create();

        $stepA = factory(FormStep::class)->create([
            'form_id' => $form->id,
            'order' => 1,
        ]);

        $stepB = factory(FormStep::class)->create([
            'form_id' => $form->id,
            'order' => 2,
        ]);

        $this->assertSameModel($stepB, $stepA->getNextStep());
        $this->assertNull($stepB->getNextStep());
    }

    /** @test **/
    public function can_get_previous_step(): void
    {
        $form = factory(Form::class)->create();

        $stepA = factory(FormStep::class)->create([
            'form_id' => $form->id,
            'order' => 1,
        ]);

        $stepB = factory(FormStep::class)->create([
            'form_id' => $form->id,
            'order' => 2,
        ]);

        $this->assertSameModel($stepA, $stepB->getPreviousStep());
        $this->assertNull($stepA->getPreviousStep());
    }

    /** @test **/
    public function can_determine_if_first_step(): void
    {
        $form = factory(Form::class)->create();

        $stepA = factory(FormStep::class)->create([
            'form_id' => $form->id,
            'order' => 1,
        ]);

        $stepB = factory(FormStep::class)->create([
            'form_id' => $form->id,
            'order' => 2,
        ]);

        $this->assertTrue($stepA->isFirst());
        $this->assertFalse($stepB->isFirst());
    }

    /** @test **/
    public function can_determine_if_last_step(): void
    {
        $form = factory(Form::class)->create();

        $stepA = factory(FormStep::class)->create([
            'form_id' => $form->id,
            'order' => 1,
        ]);

        $stepB = factory(FormStep::class)->create([
            'form_id' => $form->id,
            'order' => 2,
        ]);

        $this->assertTrue($stepB->isLast());
        $this->assertFalse($stepA->isLast());
    }

    /** @test **/
    public function can_scope_to_form()
    {
        $formA = factory(Form::class)->create();
        $stepA = factory(FormStep::class)->create(['form_id' => $formA->id]);
        $stepB = factory(FormStep::class)->create(['form_id' => $formA->id]);

        $formB = factory(Form::class)->create();
        $stepC = factory(FormStep::class)->create(['form_id' => $formB->id]);
        $stepD = factory(FormStep::class)->create(['form_id' => $formB->id]);

        $results = FormStep::forForm($formA)->get();

        $this->assertCount(2, $results);

        $this
            ->assertContainsModel($stepA, $results)
            ->assertContainsModel($stepB, $results)
            ->assertNotContainsModel($stepC, $results)
            ->assertNotContainsModel($stepD, $results);
    }

    /** @test **/
    public function can_get_as_component()
    {
        $step = factory(FormStep::class)->create([
            'name' => 'Step A Name',
        ]);

        $section = factory(FormStepSection::class)->create(['form_step_id' => $step->id]);

        $builtStep = new Step('Step A Name');
        $builtStep->fields([$section->asComponent()]);

        $this->assertEquals($builtStep, $step->asComponent());
    }
}
